/*******************************************************************************
* File Name: DRIVE_ENABLE.c  
* Version 2.20
*
* Description:
*  This file contains APIs to set up the Pins component for low power modes.
*
* Note:
*
********************************************************************************
* Copyright 2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "DRIVE_ENABLE.h"

static DRIVE_ENABLE_BACKUP_STRUCT  DRIVE_ENABLE_backup = {0u, 0u, 0u};


/*******************************************************************************
* Function Name: DRIVE_ENABLE_Sleep
****************************************************************************//**
*
* \brief Stores the pin configuration and prepares the pin for entering chip 
*  deep-sleep/hibernate modes. This function applies only to SIO and USBIO pins.
*  It should not be called for GPIO or GPIO_OVT pins.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None 
*  
* \sideeffect
*  For SIO pins, this function configures the pin input threshold to CMOS and
*  drive level to Vddio. This is needed for SIO pins when in device 
*  deep-sleep/hibernate modes.
*
* \funcusage
*  \snippet DRIVE_ENABLE_SUT.c usage_DRIVE_ENABLE_Sleep_Wakeup
*******************************************************************************/
void DRIVE_ENABLE_Sleep(void)
{
    #if defined(DRIVE_ENABLE__PC)
        DRIVE_ENABLE_backup.pcState = DRIVE_ENABLE_PC;
    #else
        #if (CY_PSOC4_4200L)
            /* Save the regulator state and put the PHY into suspend mode */
            DRIVE_ENABLE_backup.usbState = DRIVE_ENABLE_CR1_REG;
            DRIVE_ENABLE_USB_POWER_REG |= DRIVE_ENABLE_USBIO_ENTER_SLEEP;
            DRIVE_ENABLE_CR1_REG &= DRIVE_ENABLE_USBIO_CR1_OFF;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(DRIVE_ENABLE__SIO)
        DRIVE_ENABLE_backup.sioState = DRIVE_ENABLE_SIO_REG;
        /* SIO requires unregulated output buffer and single ended input buffer */
        DRIVE_ENABLE_SIO_REG &= (uint32)(~DRIVE_ENABLE_SIO_LPM_MASK);
    #endif  
}


/*******************************************************************************
* Function Name: DRIVE_ENABLE_Wakeup
****************************************************************************//**
*
* \brief Restores the pin configuration that was saved during Pin_Sleep(). This 
* function applies only to SIO and USBIO pins. It should not be called for
* GPIO or GPIO_OVT pins.
*
* For USBIO pins, the wakeup is only triggered for falling edge interrupts.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None
*  
* \funcusage
*  Refer to DRIVE_ENABLE_Sleep() for an example usage.
*******************************************************************************/
void DRIVE_ENABLE_Wakeup(void)
{
    #if defined(DRIVE_ENABLE__PC)
        DRIVE_ENABLE_PC = DRIVE_ENABLE_backup.pcState;
    #else
        #if (CY_PSOC4_4200L)
            /* Restore the regulator state and come out of suspend mode */
            DRIVE_ENABLE_USB_POWER_REG &= DRIVE_ENABLE_USBIO_EXIT_SLEEP_PH1;
            DRIVE_ENABLE_CR1_REG = DRIVE_ENABLE_backup.usbState;
            DRIVE_ENABLE_USB_POWER_REG &= DRIVE_ENABLE_USBIO_EXIT_SLEEP_PH2;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(DRIVE_ENABLE__SIO)
        DRIVE_ENABLE_SIO_REG = DRIVE_ENABLE_backup.sioState;
    #endif
}


/* [] END OF FILE */
