/***************************************************************************//**
* \file RVI_UART_SPI_UART_INT.c
* \version 4.0
*
* \brief
*  This file provides the source code to the Interrupt Service Routine for
*  the SCB Component in SPI and UART modes.
*
* Note:
*
********************************************************************************
* \copyright
* Copyright 2013-2017, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "RVI_UART_PVT.h"
#include "RVI_UART_SPI_UART_PVT.h"
#include "cyapicallbacks.h"

#if (RVI_UART_SCB_IRQ_INTERNAL)
/*******************************************************************************
* Function Name: RVI_UART_SPI_UART_ISR
****************************************************************************//**
*
*  Handles the Interrupt Service Routine for the SCB SPI or UART modes.
*
*******************************************************************************/
CY_ISR(RVI_UART_SPI_UART_ISR)
{
#if (RVI_UART_INTERNAL_RX_SW_BUFFER_CONST)
    uint32 locHead;
#endif /* (RVI_UART_INTERNAL_RX_SW_BUFFER_CONST) */

#if (RVI_UART_INTERNAL_TX_SW_BUFFER_CONST)
    uint32 locTail;
#endif /* (RVI_UART_INTERNAL_TX_SW_BUFFER_CONST) */

#ifdef RVI_UART_SPI_UART_ISR_ENTRY_CALLBACK
    RVI_UART_SPI_UART_ISR_EntryCallback();
#endif /* RVI_UART_SPI_UART_ISR_ENTRY_CALLBACK */

    if (NULL != RVI_UART_customIntrHandler)
    {
        RVI_UART_customIntrHandler();
    }

    #if(RVI_UART_CHECK_SPI_WAKE_ENABLE)
    {
        /* Clear SPI wakeup source */
        RVI_UART_ClearSpiExtClkInterruptSource(RVI_UART_INTR_SPI_EC_WAKE_UP);
    }
    #endif

    #if (RVI_UART_CHECK_RX_SW_BUFFER)
    {
        if (RVI_UART_CHECK_INTR_RX_MASKED(RVI_UART_INTR_RX_NOT_EMPTY))
        {
            do
            {
                /* Move local head index */
                locHead = (RVI_UART_rxBufferHead + 1u);

                /* Adjust local head index */
                if (RVI_UART_INTERNAL_RX_BUFFER_SIZE == locHead)
                {
                    locHead = 0u;
                }

                if (locHead == RVI_UART_rxBufferTail)
                {
                    #if (RVI_UART_CHECK_UART_RTS_CONTROL_FLOW)
                    {
                        /* There is no space in the software buffer - disable the
                        * RX Not Empty interrupt source. The data elements are
                        * still being received into the RX FIFO until the RTS signal
                        * stops the transmitter. After the data element is read from the
                        * buffer, the RX Not Empty interrupt source is enabled to
                        * move the next data element in the software buffer.
                        */
                        RVI_UART_INTR_RX_MASK_REG &= ~RVI_UART_INTR_RX_NOT_EMPTY;
                        break;
                    }
                    #else
                    {
                        /* Overflow: through away received data element */
                        (void) RVI_UART_RX_FIFO_RD_REG;
                        RVI_UART_rxBufferOverflow = (uint8) RVI_UART_INTR_RX_OVERFLOW;
                    }
                    #endif
                }
                else
                {
                    /* Store received data */
                    RVI_UART_PutWordInRxBuffer(locHead, RVI_UART_RX_FIFO_RD_REG);

                    /* Move head index */
                    RVI_UART_rxBufferHead = locHead;
                }
            }
            while(0u != RVI_UART_GET_RX_FIFO_ENTRIES);

            RVI_UART_ClearRxInterruptSource(RVI_UART_INTR_RX_NOT_EMPTY);
        }
    }
    #endif


    #if (RVI_UART_CHECK_TX_SW_BUFFER)
    {
        if (RVI_UART_CHECK_INTR_TX_MASKED(RVI_UART_INTR_TX_NOT_FULL))
        {
            do
            {
                /* Check for room in TX software buffer */
                if (RVI_UART_txBufferHead != RVI_UART_txBufferTail)
                {
                    /* Move local tail index */
                    locTail = (RVI_UART_txBufferTail + 1u);

                    /* Adjust local tail index */
                    if (RVI_UART_TX_BUFFER_SIZE == locTail)
                    {
                        locTail = 0u;
                    }

                    /* Put data into TX FIFO */
                    RVI_UART_TX_FIFO_WR_REG = RVI_UART_GetWordFromTxBuffer(locTail);

                    /* Move tail index */
                    RVI_UART_txBufferTail = locTail;
                }
                else
                {
                    /* TX software buffer is empty: complete transfer */
                    RVI_UART_DISABLE_INTR_TX(RVI_UART_INTR_TX_NOT_FULL);
                    break;
                }
            }
            while (RVI_UART_SPI_UART_FIFO_SIZE != RVI_UART_GET_TX_FIFO_ENTRIES);

            RVI_UART_ClearTxInterruptSource(RVI_UART_INTR_TX_NOT_FULL);
        }
    }
    #endif

#ifdef RVI_UART_SPI_UART_ISR_EXIT_CALLBACK
    RVI_UART_SPI_UART_ISR_ExitCallback();
#endif /* RVI_UART_SPI_UART_ISR_EXIT_CALLBACK */

}

#endif /* (RVI_UART_SCB_IRQ_INTERNAL) */


/* [] END OF FILE */
