/*******************************************************************************
* File Name: WP_VS.c  
* Version 2.20
*
* Description:
*  This file contains APIs to set up the Pins component for low power modes.
*
* Note:
*
********************************************************************************
* Copyright 2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "WP_VS.h"

static WP_VS_BACKUP_STRUCT  WP_VS_backup = {0u, 0u, 0u};


/*******************************************************************************
* Function Name: WP_VS_Sleep
****************************************************************************//**
*
* \brief Stores the pin configuration and prepares the pin for entering chip 
*  deep-sleep/hibernate modes. This function applies only to SIO and USBIO pins.
*  It should not be called for GPIO or GPIO_OVT pins.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None 
*  
* \sideeffect
*  For SIO pins, this function configures the pin input threshold to CMOS and
*  drive level to Vddio. This is needed for SIO pins when in device 
*  deep-sleep/hibernate modes.
*
* \funcusage
*  \snippet WP_VS_SUT.c usage_WP_VS_Sleep_Wakeup
*******************************************************************************/
void WP_VS_Sleep(void)
{
    #if defined(WP_VS__PC)
        WP_VS_backup.pcState = WP_VS_PC;
    #else
        #if (CY_PSOC4_4200L)
            /* Save the regulator state and put the PHY into suspend mode */
            WP_VS_backup.usbState = WP_VS_CR1_REG;
            WP_VS_USB_POWER_REG |= WP_VS_USBIO_ENTER_SLEEP;
            WP_VS_CR1_REG &= WP_VS_USBIO_CR1_OFF;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(WP_VS__SIO)
        WP_VS_backup.sioState = WP_VS_SIO_REG;
        /* SIO requires unregulated output buffer and single ended input buffer */
        WP_VS_SIO_REG &= (uint32)(~WP_VS_SIO_LPM_MASK);
    #endif  
}


/*******************************************************************************
* Function Name: WP_VS_Wakeup
****************************************************************************//**
*
* \brief Restores the pin configuration that was saved during Pin_Sleep(). This 
* function applies only to SIO and USBIO pins. It should not be called for
* GPIO or GPIO_OVT pins.
*
* For USBIO pins, the wakeup is only triggered for falling edge interrupts.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None
*  
* \funcusage
*  Refer to WP_VS_Sleep() for an example usage.
*******************************************************************************/
void WP_VS_Wakeup(void)
{
    #if defined(WP_VS__PC)
        WP_VS_PC = WP_VS_backup.pcState;
    #else
        #if (CY_PSOC4_4200L)
            /* Restore the regulator state and come out of suspend mode */
            WP_VS_USB_POWER_REG &= WP_VS_USBIO_EXIT_SLEEP_PH1;
            WP_VS_CR1_REG = WP_VS_backup.usbState;
            WP_VS_USB_POWER_REG &= WP_VS_USBIO_EXIT_SLEEP_PH2;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(WP_VS__SIO)
        WP_VS_SIO_REG = WP_VS_backup.sioState;
    #endif
}


/* [] END OF FILE */
